!> @typedef blocklist
!! @name blocklist
!! @brief List of the local blocks of a distributed matrix.
!!
!! \details This structure can be helpful when modifying a distributed
!! matrix directly on the local nodes. For example, in order to multiply
!! every column of a distributed matrix by a column dependent factor,
!! you could use the blocklist the following way:
!! @code
!!     type(blocklist) :: blocks
!!     integer :: ii, jj, jglob, jloc, bsize
!!     :
!!     call blocks%init(mygrid, descaa, "c")
!!     do ii = 1, size(blocks)
!!       call blocks%getblock(ii, jglob, jloc, bsize)
!!       do jj = 0, bsize  - 1
!!         aa(:,jloc + jj) =  aa(:,jloc + jj) * ff(jglob + jj)
!!       end do
!!     end do
!! @endcode

type :: blocklist
  private
  integer :: nn       !> the iteraton number
  integer :: nb       !> the block number
  integer :: nproc    !> number of process
  integer :: myproc   !> process in use
  integer :: srcproc  !> source ot be use for process
  integer :: nblock   !> number of blocks
  public
  integer :: init_val !> the initial value
contains
  !> Initializes the instance.
  procedure :: init => blocklist_init

  !> Returns the size of the blocklist.
  procedure :: getsize => blocklist_getsize

  !> Returns the indices (local and global) of a local block.
  procedure :: getblock => blocklist_getblock
end type blocklist

interface size
  module procedure blocklist_getsize
end interface size
