
!-----------------------------------------
!        FILE HEADER
!-----------------------------------------
! TITLE         : cours Doxygen
! PROJECT       : module use
! MODULE        : mod_initialise.f90
! URL           : ...
! AFFILIATION   : Observatoire
! DATE          : ...
! REVISION      : ... V 0.8
!> @file
!> mod_initialise.f90
!> @version
!> 2
!> @author ZAKARIA MELIANI
!> @date 11 Mars 2019
!> @brief
!! init
!> @details
!! include the initalisation procedures
!>
!!  Function  | tested
!!  ------------- | -------------
!!  initA | yes
!!  initB  | no      |
!!
!> @return Initial value for A
!> @see [https://URL]
!> @bug need tests
!--------------------------------------------
module mod_initialise
  use mod_mat
implicit none

  !> @typedef initvar
  !! @param n matrix dimension in dimension 1
  !! @param n matrix dimension in dimension 1
  !! @param val matrix value
  !! @fn  initA procedures to initialise A
  !! @see initA
  !! @fn  setzero procedures to  set A=0
  !! @see setzero
type initvar

  integer   :: n_size           !> matrix size in dimension 1
  integer   :: m_size           !> matrix size in dimension 2
  integer   :: i_ind            !> indice
contains
  PROCEDURE :: initarray  =>initA !>initial value for a matrix
  PROCEDURE :: cleanarray =>setzero
end type


contains
!> @fn initA
!> @name initA
!> @brief A initialization
!! @param [in] n dimension
!! @param [in,out] a matrix
!! @return initial value for a matrix
!! @f$ a= exp(2)+10 +j @f$
subroutine initA(n,a)
  implicit none  ! on force l'initialisation de toutes les variables
  integer, intent(in)             :: n  ! On specifie le role des arguments
  real(kind=kind(1.d0)), intent(inout)   :: a(n,n)
  integer    :: i,j
  real(kind=kind(1.d0))  :: r

  r = exp(2.)+10.
  do j = 1,n
     r = r + real(j)
     a(:,j) = r
  end do

end subroutine initA

!> @fn setzero
!> @name setzero
!> @brief A set to zero
!! @param [in] n dimension, dummy
!! @param [in,out] a matrix
!! @return null a matrix
!! @f$ a= 0 @f$
subroutine setzero(n,a)
  implicit none  ! on force l'initialisation de toutes les variables
  integer, intent(in)             :: n  ! On specifie le role des arguments
  real(kind=kind(1.d0)), intent(inout)   :: a(n,n)

   a=0
end subroutine setzero
!> @fn initB
!> @name initB
!> @brief B initialization
!! @param [in] n dimension
!! @param [in,out] b matrix
!! @f$ b= 0.2 + (0.3 + (0.4 + 0.01*ri)*ri)*ri - dj$
subroutine initB(n,b)
  implicit none  ! on force l'initialisation de toutes les variables
  integer, intent(in)       :: n
  real(kind=kind(1.d0)), intent(inout)    :: b(n,n)
  integer    :: i,j
  real(kind=kind(1.d0))  :: rj,dj,ri,d

  do j = 1,n/2-1
     rj = real(j)
     dj = (0.9 + (0.001 + 0.05*rj)*rj)*rj
     do i = 1,n
        ri = real(i)
        b(i,j) =  0.2 + (0.3 + (0.4 + 0.01*ri)*ri)*ri - dj
     end do
  end do

  d = 1./6.
  do j = n/2,n
     dj = real(j)*0.1
     do i = 1,n
        b(i,j) = i*d + dj
     end do
  end do

end subroutine initB


!> X initialization
!! @param [in] n dimension
!! @param [in,out] x vector
subroutine initX(n,x)
  implicit none  ! on force l'initialisation de toutes les variables
  integer, intent(in)       :: n
  real(kind=kind(1.d0)), intent(out)   :: x(n)
  integer    :: i
  real(kind=kind(1.d0))  :: d

  d = 0.4/7.
  do i = 1,n,4
     x(i) = real(i)*d
     x(i+1) = real(i+1)*d
     x(i+2) = real(i+2)*d
     x(i+3) = real(i+3)*d
  end do

end subroutine initX

subroutine to_test(n,a,b)
integer, intent(in)           :: n
 real(kind=kind(1.d0)), intent(in)  :: a(n)
 real(kind=kind(1.d0)), intent(out) ::b
 b=a
 call initX(n,b)
 end subroutine to_test
 
end module initialise
